@@ -1,9 +1,8 @@
-package provide custom 1.0
+package provide custom 1.63
 
-proc set_command {wsock}  {
+proc set_command {myl wsock}  {
         global w
         global nick
-        upvar l myl
        	do_custom_command $wsock $myl 
 }
 proc do_custom_command {wsock input} {
@@ -40,49 +39,12 @@ proc do_command_alias {command param} {
 	foreach com $aliaslist {
 		set scom [lindex [split $com " "] 0]
 		if ![string compare $scom $command] {
-        		set com [string trim $com " "]
-			set com [join [lrange [split $com " "] 1 end] " "]
-        		set com [string trim $com " "]
-
-			set comlist [split $com ";"]
-			foreach x $comlist {
-			set com [string trim $x]
-			set alias2 ""
-			
-			set param [string trim $param " "]
-			set list [split $param " "]
-                        set parm(1) [lindex $list 0]
-                        set parm(2) [lindex $list 1]
-                        set parm(3) [lindex $list 2]
-                        set parm(4) [lindex $list 3]
-                        set parm(5) [lindex $list 4]
-                        set parm(6) [lindex $list 5]   	
-
-			
-			set alias [split $com " "]
-                        set n 1
-			foreach word $alias {
-                               if ![string compare "-" [string range $word 0  0 ] ] {     
-					set command $word
-					substitute $myuserhost "Alias" $plist
-                                       	set alias2 [linsert $alias2 end $command]
-                                      	set n [expr $n + 1]
-                               } elseif ![string compare "#" $word] {
-                                       get_chan_name [focus]
-                                       set alias2 [linsert $alias2 end $response]
-			       } elseif ![string compare "?" $word] {
-					set x [ask_user "Enter Channel Name"] 
-					set alias2 [linsert $alias2 end $x]
-                               } else {
-                                       set alias2 [linsert $alias2 end $word]
-                               }
-                        }
-                        set param [lrange $list [expr $n-1] end]
-                        set com [join $alias2 " "]       
-puts $com
-                        fake_user_in "$com"
-			}
+        		set com [string trim $com " " ]
+			set com [join [lrange [split $com " "] 1 end] " " ]
+        		set com [string trim $com " " ]
 
+			set command $com
+			action_exec $command $myuserhost "Alias" $plist
 			return 1
 		}
 
@@ -272,17 +234,17 @@ proc do_quote {param wsock} {
 proc do_mode {param wsock } {
 	global response
 
-	set parmlist [split $param " "]
-	set chan [channel [lindex $parmlist 0] ]
-	set mode [lindex $parmlist 1]
-	set who1 [lindex $parmlist 2]
-	set who2 [lindex $parmlist 3]
-	set who3 [lindex $parmlist 4]
-
+#	set parmlist [split $param " "]
+#	set chan [channel [lindex $parmlist 0] ]
+#	set mode [lindex $parmlist 1]
+#	set who1 [lindex $parmlist 2]
+#	set who2 [lindex $parmlist 3]
+#	set who3 [lindex $parmlist 4]
 
 	
 
-	send_to_server $wsock "MODE $chan $mode $who1 $who2 $who3"
+#	send_to_server $wsock "MODE $chan $mode $who1 $who2 $who3"
+	send_to_server $wsock "MODE $param"
 
 }
 proc do_topic {param wsock} {
@@ -310,8 +272,14 @@ proc do_names {param wsock} {
 }
 
 
+proc do_script {param wsock} {
+
+	loadscript $param
+
+}
+
 proc do_save {param wsock} {
-# /save <all|ctcp|events|alias|notify|ignore|channels>
+# /save <all|ctcp|events|alias|notify|ignore|channels|url|zipper>
  
 	global rcdir
 	global aliaslist
@@ -322,6 +290,7 @@ proc do_save {param wsock} {
 	global ignore
 	global servers
 	global userlist
+	global urllist
 	
 	set ctcp 0
 	set events 0
@@ -331,8 +300,8 @@ proc do_save {param wsock} {
 	set chan 0
 	set serv 0
 	set usrs 0
-
-	
+	set url  0
+	set zipper 0	
 	
 	if [string compare $param ""] {
 		set param [string tolower $param]
@@ -346,6 +315,8 @@ proc do_save {param wsock} {
 				  set chan 1
 				  set serv 1
 				  set usrs 1
+				  set url 1
+				  set zipper 1
 				}
 		"ctcp" 		{ set ctcp 1 }
 		"events"	{ set events 1 }
@@ -355,6 +326,8 @@ proc do_save {param wsock} {
                 "channels"      { set chan 1}
                 "servers"       { set serv 1}
                 "users"         { set usrs 1}
+                "url"	        { set url 1}
+                "zipper"        { set zipper 1}
                                           
 
 		}	
@@ -415,6 +388,16 @@ proc do_save {param wsock} {
 				}
 				close $f
 			  }
+		if $url { 
+				set f [open "$rcdir/urlrc" w]	
+				foreach item $urllist {
+					puts $f $item
+				}
+				close $f
+			  }
+		if $zipper {
+				save_user_rc
+			  }
 	}
 
 
@@ -437,14 +420,14 @@ proc do_say {param wsock} {
 
 
 
-proc do_tclscript {param wsock} {
+#proc do_tclscript {param wsock} {
 #/tclscript <scriptname> [plist]
-	set name [lindex [split $param] 0]
-	set plist [lrange $param 1 end]
-	set r [$name $plist]
-	fake_user_in "/echo $r"
-
-}
+#	set name [lindex [split $param] 0]
+#	set plist [lrange $param 1 end]
+#	set r [$name $plist]
+#	fake_user_in "/echo $r"
+#
+#}
 
 proc do_play {param wsock} {
 #/play <Channel> <file> [rate]
@@ -455,12 +438,28 @@ proc do_play {param wsock} {
 
 }
 
+proc do_source {param wsock} {
+
+	source $param
+
+}
 
 proc do_exec {param wsock} {
 
-	
-	send_to_window server "\n"
-
+#        set p(1) ""
+#        set p(2) ""
+#        set p(3) ""
+#        set p(4) ""
+#        set p(5) ""
+#        set p(6) ""
+#        set x 0
+#        foreach parm $param {
+#                incr x
+#                set p($x) $parm
+#        }
+#  
+#	set r [exec $p(1) $p(2) $p(3) $p(4) $p(5) $p(6) ]
+	send_to_window server "[runpgm $param] \n"
 }
 
 proc do_ctcp {param wsock} {
@@ -574,9 +573,15 @@ proc do_load {param wsock} {
 }
 
 
+proc do_tcl {param wsock} {
+
+	set s [eval $param]
+
+}
+
+
 proc do_echo {param wsock} {
 #/echo <message>
-global chaninfo
 
 	send_to_window server "$param\n"
 }
@@ -604,31 +609,57 @@ proc do_notify {param wsock} {
 
 }
 
+proc do_dcc {param wsock} {
 
+	if [sNull $param] {
+		fake_user_in "/echo usage: /dcc <chat|send> <nickname> [filename]"
+		return
+	}
+
+	set type [string tolower [lindex [split $param " "] 0] ]
+	set rest [lrange [split $param " "] 1 end]
+
+	switch -exact -- $type {
+		"chat" 	{do_chat $rest $wsock}
+		"send"	{do_send $rest $wsock}
+	}
+
+}
 proc do_send {param wsock} {
 #/send <nickname> <filename>
 	global ip
 	global longip
 	global nick
+	global currdccdir
+	global plat
 
-	set dccbaseport [exec getport 0]
+	set dccbaseport [getport]
 	set longip $ip
 	encode_ip
 	if [string compare $param ""] {
 
 		set list [split $param]
 		set target [lindex $list 0]
-		set filename [lindex $list 1]	
+		set filename [lindex $list 1]
 		if [sNull $filename] {
-			set filename [choose_file "" "~" "open"]
+			set filename [choose_file "" $currdccdir "open"]
 			if [sNull $filename] {
 				return
 			}
+			set currdccdir [file dirname $filename]
+		}
+		if ![file exists $filename] {
+			fake_user_in "/echo DCC Filename Does Not Exist--$filename"
+			return
 		}
 		set filesize [file size $filename]
-			
-		exec dccsend.tcl 1 $target $filename $ip $dccbaseport $filesize &
+		if ![string compare $plat "unix"] {	
+			exec dccsend.tcl 1 $target $filename $ip $dccbaseport $filesize "" &
+		} else {
+			exec wish dccsend.tcl 1 $target $filename $ip $dccbaseport $filesize " " &
+		}
 		fake_user_in "/ctcp $target DCC SEND [file tail $filename] $longip $dccbaseport $filesize"
+		puts "/ctcp $target DCC SEND [file tail $filename] $longip $dccbaseport $filesize"
 	}
 }
 
@@ -639,7 +670,7 @@ proc do_chat {param wsock} {
 	global longip
 	global nick
 
-	set dccbaseport [exec getport 0]
+	set dccbaseport [getport]
 	set longip $ip
 	encode_ip
 	if [string compare $param ""] {
@@ -656,7 +687,6 @@ proc do_disconnect {param wsock} {
 
 	# close all open channel windows
 	set chanlist ""
-
 	set chanlist [.zipperserver.list get 0 end]
 	foreach item $chanlist {
 		if [is_channel $item] {
@@ -666,15 +696,10 @@ proc do_disconnect {param wsock} {
 
 	}
 
-	if $serveractive {
 	set serveractive 0
 	close $wsock
-
-	.zipperserver.controls.connect configure -state normal
-	.zipperserver.controls.disconnect configure -state disabled
 	send_to_window server "Disconneted from server.\n"
 	wm title .zipperserver "Zipper: Not Connected"
-	}
 }
 
 proc close_irc_connection {} {
@@ -689,7 +714,6 @@ proc do_edit {param wsock} {
 	global notify
 	global eventlist
 	global ctcp_message
-	global editor
 	global userlist
 	global ignore
 
@@ -699,7 +723,7 @@ proc do_edit {param wsock} {
 	set noti 0
 	set ign 0
 	set usrs 0
-	
+	set strp 0	
 	
 	if [string compare $param ""] {
 		set param [string tolower $param]
@@ -711,6 +735,7 @@ proc do_edit {param wsock} {
 				  set noti 1
 				  set ign 1
 				  set usrs 1
+				  set strp 1
 				}
 		"ctcp" 		{ set ctcp 1 }
 		"events"	{ set events 1 }
@@ -718,21 +743,23 @@ proc do_edit {param wsock} {
 		"notify"        { set noti 1 } 
 		"ignore"        { set ign 1 } 
 		"users"      	{ set usrs 1 } 
+		"startup"      	{ set strp 1 } 
 
 		}	
 		
-		if $ctcp 	{ exec $editor $rcdir/ctcprc
+		if $ctcp 	{ tked $rcdir/ctcprc
 				  init_ctcp_rec }
-		if $events 	{ exec $editor $rcdir/eventsrc
+		if $events 	{ tked $rcdir/eventsrc
 			  	  init_events }
-		if $alias 	{ exec $editor "$rcdir/aliasrc"
+		if $alias 	{ tked "$rcdir/aliasrc"
 				  init_aliases }
-		if $noti 	{  exec $editor $rcdir/notifyrc
+		if $noti 	{ tked $rcdir/notifyrc
 				  init_notify } 
-		if $ign 	{ exec $editor $rcdir/ignorerc
+		if $ign 	{ tked $rcdir/ignorerc
 				  init_ignore } 
-		if $usrs 	{ exec $editor $rcdir/usersrc
+		if $usrs 	{ tked $rcdir/usersrc
 				  init_users } 
+		if $strp 	{ tked $rcdir/startup.zipper }
 
 	}
 
@@ -740,10 +767,10 @@ proc do_edit {param wsock} {
 }
 
 proc do_recycle {a b} {
+#reload system files from disk
+#used mainly by internal editing functions
 
-	global device
-	global ip
-        exec "./ridx"
+#	fake_user_in "/echo Recycling System"
 
         init_user_rc
         init_ctcp_rec
@@ -760,16 +787,24 @@ proc do_recycle {a b} {
 
 proc do_query {myl wsock} {
         scan $myl "%s" w
-        if ![string compare "dcc" [string range $w 0 2] ] {
-                wm deiconify .zipper$w
-                raise .zipper$w
+	if ![string compare "" $w] {
+	fake_user_in "/echo QUERY: No nickame given"
+	return
+	}
+	set nw [string tolower $w]
+        if ![string compare "dcc" [string range $nw 0 2] ] {
+                wm deiconify .zipper$nw
+                raise .zipper$nw
+		focus .zipper$nw.entry
 
         } elseif [winfo exists ".zipperwin[winman $w]"] {
                 wm deiconify .zipperwin[winman $w]
                 raise .zipperwin[winman $w]
+		focus .zipperwin[winman $w].entry
         } else {
           winmannew $w "$w" "priv" "0 $w"
           insert_chan_list [string tolower $w]
+	  focus .zipperwin[winman $w].entry
           }
 
 }
@@ -810,9 +845,10 @@ proc do_action {myl wsock} {
 proc do_quit {myl wsock} {
 
         global nick
+	closealllogs
         fake_user_in "/save all"
         set myl ":$nick QUIT :$myl"
-        send_to_server $wsock $myl
+        catch [ send_to_server $wsock $myl ]
         set eventLoop "done"
         destroy .
         exit
@@ -856,15 +892,25 @@ proc do_msg {myl wsock} {
 }
 
 proc do_part {myl wsock} {
+## Thanks to Tim Howe, Parting of a channel name mixed case does not always
+## work.  The fix for this is located in util.tcl
+
 	
 	set myl [channel $myl]
 	set myl ".zipperwin[winman $myl]"
 	destroy $myl
 
-# send_to_server $wsock "PART $myl" ;# send the data to the server^M
+#### When window is destroyed, kill_window is executed which takes care
+#### of part channels and closing logs and that sort of thing
+
 }
 
 proc do_join {myl wsock} {
+
+	set fr [string index $myl 0]
+	if [string compare $fr "#"] {
+		set myl "#$myl"
+	}
         send_to_server $wsock "JOIN $myl" ;# send the data to the server^M
 }
                                 
@@ -900,11 +946,7 @@ proc do_set {myl wsock} {
 	} else {
 	set var [lindex [split $myl " "] 0]
 	set value [string trim [join [lrange [split $myl " "] 1 end] ]	" "]
-	if [expr [set index [varindex $var]] >= 0] {
-		set varlist [lreplace $varlist $index $index "$var $value"]
-	} else {
-		lappend varlist "$var $value"
-	} 
+	varset $var $value
 	}
 
 }
@@ -1051,4 +1093,28 @@ proc do_timer {myl wsock} {
 }
 
 
+proc do_sound {myl wsock} {
+# /sound sound [user/channel]
+	global appdir
+	global actwin
+
+	set sf [lindex [split $myl " "] 0]
+	set to [lindex [split $myl " "] 1]
+
+	set wavdir [decodeappdir $sf $appdir]
+	set sf [file join $wavdir $sf]
+	if [sNull $to] {
+			
+		set to [get_chan_name $actwin]		
+		if ![string compare $to "server"] {
+			playsound [file join $wavdir $sf]
+			return		
+		}
+	}
+	fake_user_in "/ctcp $to SOUND $sf"
+	playsound $sf	
+}
+
+
+
 
