@@ -1,4 +1,4 @@
-package provide server 1.0
+package provide server 1.63
 
 proc do_command {orig command param} {
 	
@@ -70,7 +70,7 @@ proc do_command {orig command param} {
 
         "PRIVMSG"       { rec_PRIVMSG $orig $param}
 
-        default         { rec_unknown $orig $param }
+        default         { rec_unknown $orig $command $param }
         }
 }
     
@@ -84,13 +84,14 @@ if $serveractive {
   if {[eof $sock]} {
         close $sock
         send_to_window server "IRC SERVER CONNECTION CLOSED!!\n\n"
-	.zipperserver.controls.connect configure -state normal
-        .zipperserver.controls.disconnect configure -state disabled
+#	.zipperserver.controls.connect configure -state normal
+#        .zipperserver.controls.disconnect configure -state disabled
         send_to_window server "Disconneted from server.\n"
         wm title .zipperserver "Zipper: Not Connected"      
         close_irc_connection
 	
   } else {
+    grab_url $l
     handle_server_message $l
   }
 }
@@ -133,24 +134,26 @@ proc rec_NOTIFY {orig param} {
         global nick
 	global notfst
 	global actwin
+	global isonlist
+	global notifywindow
 
         set list [split [string trim [string range $param [string length $nick] end] " :"] ]
 	set list [string tolower $list]
-        if [winfo exists .zippernotify] {
-                wm deiconify .zippernotify
-        } else { notify_window }
-
-	set oldlist [.zippernotify.list get 0 end]
-
-        .zippernotify.list delete 0 end
 
+	set oldlist $isonlist
+	set isonlist ""
 
 	set list [lsort -command customsort $list]
+
 	foreach name $oldlist {
 		set name [string tolower $name]
 		set x [lsearch -exact $list $name]
 		if [expr $x<0] {
-			send_to_window [get_chan_name $actwin] "$notfst$name has left IRC"
+			send_to_window server "$notfst$name has left IRC"
+			if [winfo exists $actwin] {
+#				send_to_window [get_chan_name $actwin] "$notfst$name has left IRC"
+			} 
+
 		}
 
 	}
@@ -158,12 +161,15 @@ proc rec_NOTIFY {orig param} {
 		set name [string tolower $name]
 		set x [lsearch -exact $oldlist $name]
 		if [expr $x<0] {
-			send_to_window [get_chan_name $actwin] "$notfst$name is on IRC"
+			send_to_window server "$notfst$name is on IRC"
+			if [winfo exists $actwin] {
+#				send_to_window [get_chan_name $actwin] "$notfst$name is on IRC"
+			}	
 		}
 		irc_event "NOTIFY" "N/A" $name "N/A"
-                .zippernotify.list insert end $name
+		lappend isonlist $name
         }
-
+	if $notifywindow { update_notify $isonlist }
 }
 
 
@@ -260,64 +266,6 @@ proc rec_PERSONAL {orig frommsg msg longnick dest} {
 	irc_event PRIVMSG "" $longnick ""
 }             
 
-proc rec_MODE {orig param} {
-	global nick
-	global modest
-
-	set parmlist [split $param " "]
-	set nickchan [lindex $parmlist 0]
-	set mode [string trimleft [lindex $parmlist 1] ":" ]
-	
-	set modechange [join [lrange [split $mode ""] 0 0 ] ""]
-	set modes [lrange [split $mode ""] 1 end]
-	set x 1
-	foreach item $modes {
-		set modexx $modechange$item
-		set who($x) ""
-		set who($x) [lindex $parmlist [expr $x+1 ] ]
-
-	if [is_channel $nickchan] {
-	if [sNull $who(1)] {
-		set win [winman2 $nickchan]
-
-        	if [winfo exists $win] {
-               		$win.labels.modes configure -text $mode
-        	}                               	
-		return
-	}
-	}
-	if ![string compare $nick $nickchan] {
-		send_to_window server "$modest\You have set your mode to $mode\n"
-
-	} else {
-		send_to_window $nickchan "$modest$orig sets mode $modexx to $who($x)\n"
-		set switch [lindex [split $mode ""] 0]
-		set newmode [lindex [split $mode ""] 1]
-		switch -exact -- $modexx {	
-		+o	-
-		-o	-
-		+v	-
-		-v	{togglemode $switch $who($x) $nickchan $newmode }
-		+b	-
-		-b	{fake_user_in "/echo A ban was set on $who($x)" }
-
-		default	{}	
-		}
-
-	}
-	switch -exact -- $modexx {
-		
-		+o	{ irc_event OP $nickchan $orig "$who($x)" }
-		-o	{ irc_event DEOP $nickchan $orig "$who($x)" }
-		+b	{ irc_event BAN $nickchan $orig "$who($x)" }
-		-b	{ irc_event UNBAN $nickchan $orig "$who($x)" }
-
-		default	{ irc_event MODE $nickchan $orig $mode }
-
-	}
-	incr x
-	} 
-}         
 
 
 
@@ -339,6 +287,13 @@ proc rec_NICK {orig param} {
         foreach channel $chanlist {
                 set append ""
                 set channel [string trimleft $channel "> "]
+		if ![string compare [string tolower $channel] [string tolower $orig] ] {
+			change_win_index $orig $dest
+			wm title [winman2 $dest] "Private: $dest"
+			wm iconname [winman2 $dest] "$dest"
+			set ndest [string range $dest 0 10]
+			.chanmenu.chanb1.w[winman $dest] configure -text $ndest -command "do_query $dest 0"
+		}
 		if [winfo exists .zipperwin[winman $channel].list ] {
                 set nicklist [.zipperwin[winman $channel].list get 0 end]
                 foreach nickname $nicklist {
@@ -409,14 +364,14 @@ proc rec_JOIN {orig param} {
                 focus .zipperwin[winman $chan].entry
                 send_to_window $chan "$joinst\Now Talking on $chan\n"
         } else {
-          insert_chan_list $chan
           winmannew $chan $chan "pub" "0 $chan"
+          insert_chan_list $chan
           fake_user_in "/mode $chan"
           send_to_window $chan "Now Talking on $chan\n"
         }
         } else {
         insert_nick $chan $mme
-        send_to_window $chan "$joinst\002$orig has joined $chan\002\n"
+        send_to_window $chan "$joinst\002[gfh $orig] has joined $chan\002\n"
 	process_options $orig
         }
 	irc_event  JOIN $chan $orig NA
@@ -453,9 +408,9 @@ proc rec_NOTICE {orig param} {
         if [is_on_ignore $orig "n" ] { return }
         extract_msg $param
         if [string compare "\001" [string index $msg 0] ] {
-
+		set who [get_shortnick2 $orig]
 #               send_to_window server "$notcst\--NOTICE from $orig $msg\--\n\n"
-		send_to_window [get_chan_name $actwin] "$notcst\--NOTICE from $orig $msg\--\n\n"
+		send_to_window [get_chan_name $actwin] "$notcst\--NOTICE from $who $msg\--\n\n"
         } else {
                 handle_ctcp_reply $orig $msg 
         }
@@ -486,7 +441,9 @@ proc rec_KICK {orig param} {
         send_to_window [string tolower $chan] "$kickst$orig has kicked $target from $chan: ($msg)\n"
         remove_nick $chan $target
         } else {
+		bind .zipperwin[winman $chan] <Destroy> {}
 		destroy .zipperwin[winman $chan]
+		kill_window .zipperwin[winman $chan]
 		remove_chan_list $chan
 		send_to_window server "You Were Kicked from $chan by $orig: ($msg)\n"
 		if $autorejoin { fake_user_in "/join [string tolower $chan]" }
@@ -502,12 +459,12 @@ proc rec_MOTD {orig param} {
 
 }
 
-proc rec_unknown {orig param} {
+proc rec_unknown {orig command param} {
 	
 	set list [split $param " "]
 	set list [lrange $list 1 end]
 	set param [string trimleft [join $list " "] " @=:" ]
-	send_to_window server "$param\n"
+	send_to_window server "$command $param\n"
 
 }
 
@@ -576,8 +533,10 @@ proc rec_WHOISIDLE {orig param} {
 	set time [string trimleft [lindex $list 1] ]
 	set signon [string trimleft [lindex $list 2] ]
 	set serverinfo [string trimleft [lrange $list 2 end] ":"] 
+
+	set time [secs_to_time $time]
 	if $whoisvisible {	
-		send_to_window server "[lindex $list 0]: Has been idle for $time seconds.\n"
+		send_to_window server "[lindex $list 0]: Has been idle for $time.\n"
 	}	
 
 }
@@ -603,7 +562,7 @@ proc rec_WHOISCHANNELS {orig param} {
 	extract_msg $param
 	set msg [string trim $msg " "]
 	set list [split $msg " "]
-	set msg [string trimleft [lrange $list 1 end] ":"] 
+	set msg [join [string trimleft [lrange $list 1 end] ":"]  " "]
 	if $whoisvisible {	
 		send_to_window server "[lindex $list 0]: $msg  \n"
 	}	
@@ -636,7 +595,6 @@ proc rec_JOINTOPICBY {orig param} {
 
 
 proc rec_CHANMODE {orig param} {
-puts $param
 	set plist [split $param " "]
 	set from [lindex $plist 0]
 	set chan  [lindex $plist 1]
@@ -644,7 +602,10 @@ puts $param
 	set win .zipperwin[winman $chan]
 
 	if [winfo exists $win] {
-		$win.labels.modes configure -text $modes
+		$win.labels.modes configure -state normal
+		$win.labels.modes delete 0 end
+		$win.labels.modes insert end $modes
+		$win.labels.modes configure -state disabled
 	}
 
 }
@@ -656,10 +617,11 @@ proc rec_TOPIC {orig param} {
 	set chan [lindex $plist 0]
 	set topic [string trimleft [join [lrange $plist 1 end] " "] ": "]
 	set win .zipperwin[winman $chan]
+	set sorig [get_shortnick2 $orig]
 
 	if [winfo exists $win] {
 		$win.labels.topic configure -text $topic
-		send_to_window $chan "\0033Topic changed by $orig to: \002$topic\002"
+		send_to_window $chan "\0033Topic changed by $sorig to: \002$topic\002"
 	}
 	irc_event TOPIC $chan $orig NA
 }
@@ -717,10 +679,148 @@ proc rec_LIST {orig param} {
 
 
 proc rec_CHANNELDATE {orig param} {
-puts $param
 	set channel [lindex [split $param " "] 1]
 	set inttime [lindex [split $param " "] 2]
 
 	set time [clock format $inttime]
 	fake_user_in "/echo $channel was created on $time."
 }
+
+
+proc rec_MODE {orig param} {
+
+	global nick
+	global modest
+
+
+        set parmlist [split $param " "]
+        set nickchan [lindex $parmlist 0]
+        set mode [split [string trimleft [lindex $parmlist 1] ":" ]  ""]     
+	set target [lindex $parmlist 2]
+
+
+	if ![winfo exists [winman2 $nickchan] ] {
+		return
+	}
+	
+	if [sNull $target] {
+		set target $nickchan
+	}
+
+	if ![string compare $nickchan $nick] {
+		if ![string compare $target $nick] {
+			rec_SETUSERMODE $nickchan $mode $parmlist
+			return
+		}
+	} 
+
+	if ![string compare $nickchan $target] {
+		rec_SETCHANMODE $nickchan $mode $parmlist
+		return
+	}
+	if [string match "*l*" [join $mode "" ] ] {
+		rec_SETCHANMODE $nickchan $mode $parmlist
+		return
+
+	}
+	if [string match "*k*" [join $mode "" ] ] {
+		rec_SETCHANMODE $nickchan $mode $parmlist
+		return
+
+	}
+	rec_SETCHANUSERMODE $orig $nickchan $mode $parmlist
+
+}
+
+proc rec_SETUSERMODE {nick mode parmlist} {
+	global modest	
+	set mode [join $mode ""]
+	if ![sNull $mode] {
+		send_to_window server "$modest\You have set your mode to $mode\n"
+	}
+}
+
+proc rec_SETCHANMODE {chan mode parmlist} {
+	global modest
+	set mode [join $mode ""]
+	if ![sNull $mode] {
+		set_chan_mode $chan $mode
+	}
+
+}
+
+proc rec_SETCHANUSERMODE {orig nickchan mode parmlist} {
+	global nick
+	global modest 
+
+	set who(1) ""
+        set who(1) "$nickchan:[lindex $parmlist 2]"
+        set who(2) ""
+        set who(2) "$nickchan:[lindex $parmlist 3]"
+        set who(3) ""
+        set who(3) "$nickchan:[lindex $parmlist 4]"
+        set who(4) ""
+        set who(4) "$nickchan:[lindex $parmlist 5]"
+        set who(5) ""
+        set who(5) "$nickchan:[lindex $parmlist 6]"
+        set who(6) ""
+        set who(6) "$nickchan:[lindex $parmlist 7]"
+                                    
+
+	set x 1
+	set modechange "+"
+	foreach item $mode {
+		if [expr $item=="+" || $item=="-"] {
+                        set modechange $item
+                        set x 1
+                        continue
+                }         
+		append who($x) ":$modechange$item"
+                incr x       
+	}
+
+
+
+	set ml [list $who(1) $who(2) $who(3) $who(4) $who(5) $who(6) ]
+        foreach user $ml {
+                set f [split $user ":"]
+                set nickchan [lindex $f 0]
+                set target [lindex $f 1]
+                set mode [lrange $f 2 end]
+
+		if ![sNull $mode] {
+			send_to_window $nickchan "$modest$orig sets mode $mode $target\n"
+               		 foreach modex $mode {
+                		set switch  [lindex [split $modex ""] 0]
+                		set newmode [lindex [split $modex ""] 1]
+                		switch -exact -- $modex {      
+				
+					+v      -
+                			-v      -
+                			+o      -
+                			-o      {togglemode $switch $target $nickchan $newmode }
+                			+b      {fake_user_in "/echo A ban was set on $target" }
+                			-b      {fake_user_in "/echo A ban was removed on $target" }
+
+                			default {}
+                		}
+                		switch -exact -- $modex {
+                			+o      { irc_event OP $target $orig "$target" }
+                			-o      { irc_event DEOP $target $orig "$target" }
+                			+b      { irc_event BAN $target $orig "$target" }
+                			-b      { irc_event UNBAN $target $orig "$target" }
+
+                			default { irc_event MODE $target $orig $modex }
+
+                		}                                                          
+
+				
+			}
+		}            
+
+	}
+}
+
+
+
+
