@@ -1,4 +1,4 @@
-package provide util 1.0
+package provide util 1.631
 
 
 
@@ -58,6 +58,30 @@ proc get_shortnick lorig {
 
 
 }
+proc get_shortnick2 {orig} {
+
+        if { [string first "!" $orig] > -1 } {
+                set sorig [string range $orig 0 [string first "!" $orig] ]
+                set sorig [string trimright $sorig "!"]
+                return $sorig
+         }
+
+}
+
+proc gfh {orig} {
+# Return a formatted version of the user host
+	
+	set t [split $orig "!"]
+	set nick [lindex $t 0]	
+	set mask [lindex $t 1]
+	if [sNull $mask] {
+		set form "$nick"
+	} else {
+		set form "$nick \($mask\)"
+	}
+	return $form
+}
+
 
 proc get_nick_from_hostmask hostmask {
 
@@ -204,6 +228,27 @@ proc insert_chan_list chan {
         foreach element $clist {
                 .zipperserver.list insert end $element
         }
+	set w w[winman $chan]
+	set schan [string range $chan 0 10]
+	if ![winfo exists .chanmenu.chanb1.$w] {
+		set isdcc [string range $chan 0 2]
+		if ![string compare $isdcc "DCC" ] {
+			set dnick [lindex [split $chan ":"] 1]
+			set chan dcc[get_dcc_sock_from_nick $dnick]
+		}
+		set clist [split $chan "" ]
+		set nlist ""
+		foreach c $clist {
+			switch -exact -- $c  {
+				"\[" {set c "\\$c" }
+				"\{" {set c "\\$c" }
+			}
+			lappend nlist $c
+		}
+		set chan [join $nlist "" ]
+		button .chanmenu.chanb1.$w -width 9 -font ZipperN -text $schan -command "do_query $chan 0"
+	}
+	pack .chanmenu.chanb1.$w -side top 
 	}
 }                                             
 
@@ -219,6 +264,10 @@ proc remove_chan_list chan {
         foreach name $thelist {
                 .zipperserver.list insert end $name
         }
+	set w w[winman $chan]
+	if [winfo exists  .chanmenu.chanb1.$w ] {
+		pack forget .chanmenu.chanb1.$w 
+	}
 }
 
 
@@ -279,14 +328,29 @@ proc is_active chan {
 
 proc customsort {a b} {
 
+	set x [string index $a 0]
+	set y [string index $b 0]
+	
+	switch -exact -- $x {
+		"@"	{set a "A$a" }
+		"+"	{set a "B$a" }
+		"_"	{set a "Z$a" }
+		default {set a "G$a" }
 
+	}
+	switch -exact -- $y {
+		"@"	{set b "A$b" }
+		"+"	{set b "B$b" }
+		"_"	{set b "Z$b" }
+		default {set b "G$b" }
+
+	}
         return [string compare [string tolower $a] [string tolower $b] ]
 
 }    
 proc init_user_rc {} {
 	global rcdir
         global installpath
-        global version
         global nick
         global altnick
         global username
@@ -296,11 +360,22 @@ proc init_user_rc {} {
         global shortnick
         global server
         global port
-        global editor
 	global textfg
 	global textbg
 	global device
-	
+	global notifywindow
+	global appdir
+	global webbrowser
+	global autoacceptchat
+	global autoacceptsend
+	global dccsavdir
+	global logdir
+	global logchannels
+	global logmsg
+	global logdcc
+	global serverpass
+	global dccapplist
+			
         set f [open $rcdir/zipperrc r]
 
         while 1 {
@@ -311,7 +386,7 @@ proc init_user_rc {} {
                 }
                 set comlist [split $l " "]
                 set command [lindex $comlist 0]
-                set param [string trimright [lrange $comlist 1 end] " " ]
+                set param [join [string trimright [lrange $comlist 1 end] " " ] " "]
                 set $command $param
 
         }
@@ -371,6 +446,32 @@ proc init_servers {} {
 
 
 
+}                  
+proc init_url {} {
+
+        global urllist 
+	global rcdir
+
+        set urllist ""
+        set f [open $rcdir/urlrc r]
+
+
+        while 1 {
+        set l [gets $f]
+                if [ eof $f ] {
+                        close $f
+                        return 1
+                }
+#                if [string compare "#" [string index $l 0] ] {
+                        set urllist [linsert $urllist end $l]
+#                }
+
+        }
+
+
+
+
+
 }                  
 proc init_users {} {
 
@@ -541,11 +642,11 @@ proc check_notify {} {
 
 proc init_check_rc {} {
         global installpath
+	global rcdir
 
-        if ![file exists "~/.zipper"] {
-                puts stdout "Could not find zipper resource files."
-                puts stdout "Please cd to $installpath and execute the"
-                puts stdout "installrc script."
+        if ![file exists $rcdir ] {
+		wm withdraw .
+                bgerror "Could not find zipper resource files.  Please cd to $installpath and execute the installrc script."
                 exit
         }
 
@@ -560,7 +661,6 @@ proc send_to_server {wsock l} {
 
   global serveractive
   global bytefloodlimit
-
   set x [string length $l]
   if $x>$bytefloodlimit {
 	 set m [tk_dialog .userflood "Flood Limit Warning" "Flood Limit Exceeding..Hit Ok to send anyway." "error" 0 "ok"  "cancel" ]
@@ -594,7 +694,7 @@ proc fake_user_in {input} {
   if ![string compare "/" [string index $l 0] ] {
 	
         	set l [string trimleft $l " /"]
-       		set_command $esvrSock
+       		set_command $l $esvrSock
   } else {
 
         if ![string compare ".zipperserver" $w] {
@@ -636,21 +736,26 @@ proc read_user_in {wsock w} {
 #default is to send to current channel
 
   if [sNull $l] return
-  if ![string compare "/" [string index $l 0] ] {
-        set l [string trimleft $l "/"]
-        if [string compare "" $l] {
-                set_command $wsock
+
+  set lines [split $l "\n"] 
+
+  foreach line $lines	{
+  if ![string compare "/" [string index $line 0] ] {
+        set line [string trimleft $line "/"]
+        if [string compare "" $line] {
+                set_command $line $wsock
         }
   } else {
 
         if ![string compare "server" $w] {
         } else {
-	  if $chef { set l [exec chef.sh $l] }	
-          send_to_window $chan "> $l\n"
-          set l ":$nick PRIVMSG $chan :$l"
-          send_to_server $wsock $l ;# send the data to the server
+	  if $chef { set line [exec chef.sh $line] }	
+          send_to_window $chan "> $line\n"
+          set line ":$nick PRIVMSG $chan :$line"
+          send_to_server $wsock $line ;# send the data to the server
           }
   }
+ }
 
 
 }                            
@@ -667,6 +772,7 @@ proc set_chan_activity {status win} {
 	set type $chaninfo($win,type)
 	set tnick $chaninfo($win,nickchan)
 
+### the old way for channel list
 	if ![string compare $type "dcc" ] { 
 		set win "DCC:$tnick"
 	}
@@ -680,15 +786,19 @@ proc set_chan_activity {status win} {
         if [string match "server*" $win] { return }
 
         if $status {
-                remove_chan_list $win
-                remove_chan_list "> $win"
-                insert_chan_list "> $win"
-		wm iconbitmap $zwin "@$installpath/zipperwinicon2.bmp"
+#                remove_chan_list $win
+#                remove_chan_list "> $win"
+#                insert_chan_list "> $win"
+		wm iconbitmap $zwin "@$installpath/zipperwinicon2.xbm"
+		set w w[winman $win]
+		.chanmenu.chanb1.$w configure -foreground red
         } else {
-                remove_chan_list "$win"
-                remove_chan_list "> $win"
-                insert_chan_list "$win"
-		wm iconbitmap $zwin "@$installpath/zipperwinicon.bmp"
+#                remove_chan_list "$win"
+#                remove_chan_list "> $win"
+#                insert_chan_list "$win"
+		wm iconbitmap $zwin "@$installpath/zipperwinicon.xbm"
+		set w w[winman $win]
+		.chanmenu.chanb1.$w configure -foreground black
         }
 
 }
@@ -708,7 +818,7 @@ global actwin
 
 
 
-proc substitute {fromnick fromchannel line} {
+proc substitute {newcommand fromnick fromchannel line} {
 
 
 	global nick
@@ -727,7 +837,6 @@ proc substitute {fromnick fromchannel line} {
 	set level [get_level $fromnick]		
 	set fromuser $fromnick
 	set fromnick [get_nick_from_hostmask2 $fromnick]
-	upvar command newcommand
 	set newcommand [string trim $newcommand " "]	
 	set oldcommand $newcommand
 	
@@ -735,6 +844,7 @@ proc substitute {fromnick fromchannel line} {
 	set newlist ""
 	foreach word $list {
 		set check [string tolower $word]
+
 		if ![string compare "-$" [string range $check 0 1] ] {
 			set check [string range $check 1 end]
 			set pc [string range $check 1 4]
@@ -756,6 +866,12 @@ proc substitute {fromnick fromchannel line} {
 			set check [string range $check 2 end]
 			set check [funcvalue $check $fromuser $fromchannel $line]
 			set newlist [linsert $newlist end $check]
+		} elseif ![string compare "#" $check ] {
+                                       set cn [get_chan_name [focus] ]
+                                       set newlist [linsert $newlist end $cn]
+                } elseif ![string compare "?" $check] {
+                                        set x [ask_user "Question" "Enter Channel Name" ]
+                                        set newlist [linsert $newlist end $x] 
 		} else {
 			set newlist [linsert $newlist end $word]
 
@@ -764,15 +880,16 @@ proc substitute {fromnick fromchannel line} {
 	set list $newlist
 
 	set newcommand [join $newlist " "]
-
+	return $newcommand
 }
 
 
 
-proc do_menu_command {win index type} {
-global userpopups
+#proc do_menu_command {win index type} {
+proc do_menu_command {win com type } {
+#global userpopups
 global nick
-global parm1
+
 	set win ".[lindex [split $win "."] 1]"
 
 	switch -exact -- $type {
@@ -781,12 +898,10 @@ global parm1
 		userlist	-
 		notify		{set parm1 [string trimleft [$win.list get active] "@+"]}
 	}
-	set index [expr $index - 1]
-
-	set com [lindex $userpopups($type) $index]
+#	set index [expr $index - 1]
+#	set com [lindex $userpopups($type) $index]
 	set command $com
-	substitute $nick NA $parm1
-	fake_user_in $command
+	action_exec $command $nick NA $parm1
 
 }
 
@@ -831,7 +946,6 @@ proc varindex {varname} {
 
 proc varset {var value} {
   	global varlist   
-      
         if [expr [set index [varindex $var]] >= 0] {
                 set varlist [lreplace $varlist $index $index "$var $value"]
         } else {
@@ -846,7 +960,7 @@ proc varvalue {varname} {
 
 	global varlist
         set index [varindex $varname]
-	set value [string trim [join [lrange [split [lindex $varlist $index] " "] 1 end] ]  " "] 
+	set value [string trim [join [lrange [split [lindex $varlist $index] " " ] 1 end] " " ]  ] 
         return $value 
 
 }
@@ -873,12 +987,12 @@ proc funcvalue {function fromuser channel line} {
 	foreach parm $parmlist {
 		incr x
 		if ![string compare "-" [string range $parm 0  0 ] ] {
-                	set command $parm
-                       	substitute $fromuser $channel $line
-                        set parm $command
+                       	set parm [substitute $parm $fromuser $channel $line]
                 }
-		set p($x) $parm
+		set p($x) [string trim $parm]
 	}
+	set cmdline "$p(1) $p(2) $p(3) $p(4) $p(5) $p(6)" 
+	set plist "$p(2) $p(3) $p(4) $p(5) $p(6)" 
 
 	switch $function {
 	"level" { 	set whoisvisible 0
@@ -887,20 +1001,32 @@ proc funcvalue {function fromuser channel line} {
 			tkwait variable whoisvisible
 			return [get_level $whoisuser] 
 		 }
-	"longip" { return [exec encodeip $p(1) ]  }
-	"shortip" { return [exec decodeip $p(1) ]  }
-
-	"date" {  return [exec date]  }
-	"add"	{ return [expr $p(1) + $p(2) ] }	
+	"longip" { return [encodeip $p(1) ]  }
+	"shortip" { return [decodeip $p(1) ]  }
+	"date" {  return [date]  }
+	"add"	{ return [expr $p(1) + $p(2) ] }
+	"cat"	{ 	set c "$p(1)$p(2)" 
+			return $c }
 	"sub"	{ return [expr $p(1) - $p(2) ]  }
-	"rtime"  { return [exec pingtime] }
-	"run"	{ return [exec $p(1) $p(2) $p(3) $p(4) $p(5) $p(6) ] }
+	"rtime"  { return [clock seconds] }
+	"run"	{ return [runpgm $cmdline] }
 	"none"	{ return }
-	"tcl"   {	set plist "$p(2) $p(3) $p(4) $p(5) $p(6)" 
-			$p(1) $plist } 
-	"time"  { set date [exec date]
-		  set time [lindex [split $date " "] 4]
-		  return $time }
+	"strange"	{ set start $p(1)
+		  set end   $p(2)
+		  set str   $p(3)
+		  set s [string range $p(3) $start $end]
+		  return $s}
+	"prange" {	set p(1) [expr $p(1)-1]
+			if [string compare $p(2) "end"] {
+				set p(2) [expr $p(1)-1]
+			}
+			set linex [join $line]
+			set xlist [split $linex " "]
+			set pr [lrange $linex $p(1) $p(2)]
+			return [join $pr " "]
+		}
+	"tcl"   { return [eval $cmdline] } 
+	"time"  { return [time] }
 	"window"	{ return [get_chan_name $actwin] }
 
 	"chanlist" { 	set chan $p(1)
@@ -981,7 +1107,10 @@ proc playsystembell {} {
 }
 
 
-proc loadstartupscript {script} {
+proc loadscript {script} {
+	global nick
+
+	if [file exists $script] {
 	set f [open $script r]
 	 while 1 {
      	   set l [gets $f]
@@ -989,7 +1118,11 @@ proc loadstartupscript {script} {
                         close $f
                         return 1
                 }
-	   fake_user_in "$l"
+	   set command $l
+	   action_exec $command $nick "N/A" ""
+	}
+	} else {
+		fake_user_in "/echo $script not found!"
 	}
 
 
@@ -998,9 +1131,12 @@ proc loadstartupscript {script} {
 
 
 proc is_channel {channel} {
+## Tim Howe submitted the change to allow mixed case channel names
+## to function properly on parts.
+## the change is a simply string tolower in the first set statement
 global chaninfo
 
-	set channel [string trimleft $channel "> "]
+	set channel [string trimleft [string tolower $channel] "> "]
 	if [sNull [array names chaninfo "$channel,name"] ] {
                 return 0
         } else {
@@ -1028,7 +1164,7 @@ proc center {t  xoff yoff} {
             - [winfo vrootx [winfo parent $t]]]
     	set y [expr [winfo screenheight $t]/2 - [winfo reqheight $t]/2 \
             - [winfo vrooty [winfo parent $t]]]         
-	
+           
 	set x [expr $x+$xoff] 
 	set y [expr $y+$yoff] 
 	wm geom $t +$x+$y  
@@ -1054,10 +1190,12 @@ proc set_popups {} {
 	set mchan .zipperserver.menu.channelpopup      
 	set mquery .zipperserver.menu.querypopup      
 	
-	menu $muser
-	menu $mnoti
-	menu $mchan
-	menu $mquery
+	menu $muser -tearoff no -font ZipperMenu
+	menu $mnoti -tearoff no -font ZipperMenu
+	menu $mchan -tearoff no -font ZipperMenu
+	menu $mquery -tearoff no -font ZipperMenu
+
+	set submenu 0
 
 	set f [open $rcdir/popuprc r]
 
@@ -1087,9 +1225,11 @@ proc set_popups {} {
                		 } else {
                 	set lab [lrange [split $l " "] 0 0 ]
                 	set com [join [lrange [split $l " "] 1 end] " "]
-			set mc "do_menu_command \[focus\] \[$m index active\] $t"
-			set userpopups($t) [linsert $userpopups($t) end "$com"] 
-                	$m add command -label $lab -command $mc 
+                	set com [join [split $com "-"] "-\\"]
+#			set mc "do_menu_command \[focus\] \[$m index active\] $t"
+			set mc "do_menu_command \[focus\] \"$com\" $t"
+#			set userpopups($t) [linsert $userpopups($t) end "$com"] 
+                	$m add command -label $lab -command $mc
                 	}                        
 		}
 
@@ -1168,7 +1308,7 @@ proc chan_exists {channel} {
 	if [string match $channel "server"] {return 1}
 	set list [.zipperserver.list get 0 end]
 	foreach item $list {
-		set item [string trim $item "> "]
+		set item [string tolower [string trim $item "> "] ]
 		if ![string compare $item $channel ] {return 1}
 	}
 	return 0
@@ -1287,3 +1427,468 @@ proc kill_timer {num} {
 	global timerarray
 	unset timerarray($num) 
 }
+
+
+proc update_notify {list} {
+
+global notifywindow 
+
+        if [winfo exists .zippernotify] {
+
+        } else { notify_window }
+        .zippernotify.list delete 0 end       
+
+	foreach item $list {
+		.zippernotify.list insert end $item
+	}
+}
+
+proc toggle_notify_win {} {
+global notifywindow
+global isonlist
+
+	if !$notifywindow { destroy .zippernotify 
+	} else {	
+		notify_window
+		update_notify $isonlist
+	}	
+
+}
+
+
+proc minimize {w} {
+
+	set window [lindex [split $w "."] 1]
+	wm iconify .$window
+
+}
+
+proc create_fonts {} {
+
+	font create ZipperN -family helvetica -size 12 -weight normal -slant roman 
+	font create ZipperB -family helvetica -size 12 -weight bold -slant roman
+	font create ZipperBBar -family helvetica -size 10 -weight normal
+	font create ZipperMenu -family helvetica -size 10 -weight normal
+	font create ZipperTopic -family Times -size 12 -weight normal
+
+
+}
+
+proc secs_to_time {secs} {
+
+	set time ""
+	set hours [expr int($secs/3600) ]
+	set hourc [expr $hours*3600]
+	set secr [expr $secs-$hourc]
+	set min [expr int($secr/60)]
+	set minc [expr int($min*60)]
+	set secr [expr $secr-$minc]
+
+
+	if $hours {
+                set time "$hours Hours, "
+	}
+
+	if $min {
+                set time "$time $min Minutes,"
+        }
+        set time "$time $secr Seconds"
+                                          
+	return $time
+}
+
+
+
+proc playsound {sf} {
+	global dccapplist
+
+	set pgm [decodeapp $sf $dccapplist]
+
+	if [file exists $sf] {
+		catch {set r [ runpgm "$pgm $sf > /dev/null &"] }
+	} else {
+		send_to_window server "No Such File: $sf \n"
+	}
+}
+
+
+proc grab_url {input} {
+global urllist
+
+	set input1 [string tolower $input]
+	set s [string first "http://" $input1]
+
+	if $s>=0 {
+
+		set url [string range $input $s end]
+		set e [string first " " $url]
+		if $e<0 {
+			set e [string length $url]	
+		}
+		set url [string range $url 0 $e]
+		set x [lsearch -exact $urllist $url]
+		if $x==-1 {
+			if [lsearch -exact $urllist $url]<0 {
+				lappend urllist $url
+				if [winfo exists .zipperurl] {
+					.zipperurl.box.list insert end $url
+				}
+			}
+		}
+	}
+
+
+}
+
+proc launch_url {url} {
+	global webbrowser
+	fake_user_in "/echo Launching $url"
+
+	set x [ catch { runpgm "$webbrowser -remote \"openURL($url)\" &"   } ct ]
+
+
+	if $x {
+		"Could not connect to remote $webbrowser, attempting to start."
+		set x [ catch {exec $webbrowser $url & } ct ]
+	}
+	if $x {
+		puts "Could not start $webbrowser."
+		bgerror $ct
+	}
+
+}
+
+
+proc set_chan_mode {chan mode} {
+#  reset mode array
+array set modes {
+	s ""
+	p ""
+	m ""
+	t ""
+	i ""
+	n ""
+	m ""
+	l ""
+	k ""
+}
+
+## Get Current Modes
+	set cmodes [[winman2 $chan].labels.modes get]
+	set ar [array names modes]
+	foreach item [split $cmodes ""] {
+		if [string match *$item* $ar] {
+			set modes($item) $item
+		}	
+
+	}
+	set cmodes $mode
+	set v "+"
+	foreach item [split $cmodes ""] {
+		if [string match *$item* "+"] {
+			set v "+"
+		}
+		if [string match *$item* "-"] {
+			set v "-"
+		}
+
+		if [string match *$item* $ar] {
+			switch -exact -- $v {
+			"+"	{ set modes($item) $item }
+			"-"	{ set modes($item) "" }
+			}
+		}
+	}
+	set mode ""
+	foreach i $ar {
+			set mode $mode$modes($i)
+	}	
+
+	if [winfo exists [winman2 $chan] ] {
+	
+		[winman2 $chan].labels.modes configure -state normal
+		[winman2 $chan].labels.modes delete 0 end
+		[winman2 $chan].labels.modes insert end $mode
+		[winman2 $chan].labels.modes configure -state disabled
+	}
+}
+
+
+
+
+proc runpgm {cmdline} {
+
+	set s [eval exec $cmdline]
+	return $s
+}
+
+proc date {} {
+	set time [clock seconds]
+	return [clock format $time -format %D]
+
+}
+proc time {} {
+	set time [clock seconds]
+	return [clock format $time -format %T]
+
+}
+
+
+proc change_win_index {oldnick newnick} {
+global chaninfo
+global chanrev
+
+	set oldnick [string tolower $oldnick]
+	set newnick [string tolower $newnick]
+	set index $newnick
+
+	set chaninfo($index,name) $chaninfo($oldnick,name) 
+	set chaninfo($index,type) $chaninfo($oldnick,type)
+        set chaninfo($index,nickchan) $chaninfo($oldnick,nickchan)
+        set chaninfo($index,stat) $chaninfo($oldnick,stat)
+        set chaninfo($index,history)  $chaninfo($oldnick,history)
+        set chaninfo($index,historyidx)  $chaninfo($oldnick,historyidx)
+        set chaninfo($index,winnumber) $chaninfo($oldnick,winnumber)
+        set chaninfo($index,socket) $chaninfo($oldnick,socket)
+     	set chaninfo($index,logging) $chaninfo($oldnick,logging)   
+     	set chaninfo($index,filedesc) $chaninfo($oldnick,filedesc)   
+
+	if $chaninfo($oldnick,logging) {closelog $oldnick }
+     	set chaninfo($index,logfile) $chaninfo($oldnick,logfile)   
+
+	set chanrev($chaninfo($index,name)) $index                
+
+	unset chaninfo($oldnick,name)
+	unset chaninfo($oldnick,type)
+	unset chaninfo($oldnick,nickchan)
+	unset chaninfo($oldnick,stat)
+	unset chaninfo($oldnick,history)
+	unset chaninfo($oldnick,historyidx)
+	unset chaninfo($oldnick,winnumber)
+	unset chaninfo($oldnick,socket)
+	unset chaninfo($oldnick,logging)
+	unset chaninfo($oldnick,filedesc)
+	unset chaninfo($oldnick,logfile)
+	
+	if $chaninfo($index,logging) {openlog $index} 
+	
+}
+
+
+
+proc action_exec {action fromnick fromchannel line} {
+
+## Support for "Old Style" commands and "New TCL Code" Commands
+
+#	set action [substitute $action $fromnick $fromchannel $line]
+
+	set action [string trim $action]
+	set type [string index $action 0]
+	switch -exact -- $type {
+
+	"/"	{ action_exec_old $action $fromnick $fromchannel $line}
+	"\{"	{ action_exec_tcl $action $fromnick $fromchannel $line}
+
+	}
+
+		
+
+}
+
+
+proc action_exec_old {action fromnick fromchannel line} {
+
+	set l [split $action ";"]
+	foreach command $l {
+		set command [substitute $command $fromnick $fromchannel $line]
+		fake_user_in $command
+	}                              
+
+
+}
+
+proc action_exec_tcl {action fromnick fromchannel line} {
+	set action [string trim $action "\{\}" ]
+
+	set l [split $action ";"]
+	foreach command $l {
+		set action [substitute $command $fromnick $fromchannel $line]
+		eval $action
+	}	
+}
+
+proc irc {command} {
+
+	fake_user_in $command
+}
+
+
+proc pick_color {} {
+global mirccolors
+global tmpcolor
+
+	set w .colorpick
+	toplevel $w
+	frame $w.f1
+	frame $w.f2
+	frame $w.f3
+	frame $w.f4
+	pack $w.f1 -side top
+	pack $w.f2 -side top
+	pack $w.f3 -side top
+	pack $w.f4 -side top
+
+	for {set i 0} {$i<4} {incr i} {
+		button $w.f1.color$i -background $mirccolors($i) -command "destroy .colorpick; set tmpcolor $i"
+		pack $w.f1.color$i -side left
+	}
+	for {set i 4} {$i<8} {incr i} {
+		button $w.f2.color$i -background $mirccolors($i) -command "destroy .colorpick; set tmpcolor $i"
+		pack $w.f2.color$i -side left
+	}
+	for {set i 8} {$i<12} {incr i} {
+		button $w.f3.color$i -background $mirccolors($i) -command "destroy .colorpick; set tmpcolor $i"
+		pack $w.f3.color$i -side left
+	}
+	for {set i 12} {$i<16} {incr i} {
+		button $w.f4.color$i -background $mirccolors($i) -command "destroy .colorpick; set tmpcolor $i"
+		pack $w.f4.color$i -side left
+	}
+
+	tkwait window .colorpick
+	return $tmpcolor
+}
+
+
+proc openlog {logname} {
+global chaninfo
+global logdir
+	
+	set name [string tolower $logname]
+	if ![string compare $chaninfo($logname,type) "dcc" ] {
+		set name "DCC_[get_dcc_nick_from_socket $chaninfo($logname,socket)]"
+	}
+	set fname [file join $logdir $name]
+	if ![file exists $logdir] {
+		file mkdir $logdir
+	}	
+
+	set fp [open $fname "a+"]
+	set date [date]
+	set time [time]
+	puts $fp "Logging Started for $name, on $date at $time"
+	set chaninfo($logname,logfile) $fp
+}
+
+proc closelog {logname} {
+global chaninfo
+
+	set logname [string tolower $logname]
+	set name $logname
+
+	set x ""
+	catch { set x $chaninfo($logname,type) }
+	if ![string compare $x "dcc" ] {
+		set name "DCC_[get_dcc_nick_from_socket $chaninfo($logname,socket)]"
+	}
+
+	set fp $chaninfo($logname,logfile)
+	if ![sNull $fp] {
+	set date [date]
+	set time [time]
+	puts $fp "Logging Ended for $name, on $date at $time"
+	close $fp
+	set chaninfo($logname,logfile) ""
+	}
+}
+proc closealllogs {} {
+global chaninfo
+	set nlist [array names chaninfo "*logfile"]
+	foreach log $nlist {
+		closelog [lindex [split $log ","] 0]
+
+	}
+
+}
+
+proc add_to_log {file text} {
+	puts -nonewline $file $text
+
+
+}
+
+
+proc toggle_logging {w} {
+	global chaninfo
+	global chanrev
+
+	set win [string tolower $chanrev($w) ]
+	if ![sNull $chaninfo($win,logfile) ] {
+		closelog $win
+	} else {
+		openlog $win
+	}
+	
+
+}
+proc decodeapp {file apps} {
+       
+	set xl [split $file "."]
+        set filetype [lindex $xl end]
+
+        set al [split $apps ","]
+        foreach item $al {
+                set typ [lindex [split $item "="] 0]
+                set pgm [lindex [split $item "="] 1]
+                if [string compare $typ $filetype]==0 {
+                        return $pgm
+                }
+        }
+	set pgm ""
+	return $pgm
+
+
+}
+proc showfile {file apps} {
+
+
+	set pgm [decodeapp $file $apps]	
+	if ![sNull $pgm] {
+        	exec $pgm $file &
+	}
+
+}       
+proc decodeappdir {file dirs} {
+global dccsavdir
+       
+	set xl [split $file "."]
+        set filetype [lindex $xl end]
+        set al [split $dirs ","]
+        foreach item $al {
+                set typ [lindex [split $item "="] 0]
+                set dir [lindex [split $item "="] 1]
+                if [string compare $typ $filetype]==0 {
+                        return [expandhome $dir]
+                }
+        }
+	set dir $dccsavdir
+	return $dir
+
+
+}
+
+
+proc expandhome {dir} {
+	global homedir
+	set homepath $dir
+	set dirroot [file dirname ~]
+	set dlist [file split $dir]
+	set x [lsearch -exact $dlist ~]
+	if $x>=0 {
+		set dlist [lreplace $dlist $x $x [file join $dirroot $homedir] ]
+	}
+	set homepath [join $dlist "/"]
+	return $homepath
+
+
+}
